@@ -24,6 +24,7 @@
 #include <glib/gstdio.h>
 
 #include "gsm-util.h"
+#include "gsm-autostart-app.h"
 #include "gsm-client.h"
 
 #include "gsm-session-save.h"
@@ -83,7 +84,7 @@ save_one_client (char            *id,
 
         discard_exec = g_key_file_get_string (keyfile,
                                               G_KEY_FILE_DESKTOP_GROUP,
-                                              "X-GNOME-Autostart-discard-exec",
+                                              GSM_AUTOSTART_APP_DISCARD_KEY,
                                               NULL);
         if (discard_exec) {
                 g_hash_table_insert (data->discard_hash,
@@ -183,7 +184,7 @@ gsm_session_clear_one_client (const char *filename,
 
                 discard_exec = g_key_file_get_string (key_file,
                                                       G_KEY_FILE_DESKTOP_GROUP,
-                                                      "X-GNOME-Autostart-discard-exec",
+                                                      GSM_AUTOSTART_APP_DISCARD_KEY,
                                                       NULL);
                 if (!discard_exec)
                         goto out;
