@@ -28,19 +28,20 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include <gio/gio.h>
 #include <glib/gi18n.h>
 
 #include "gsm-xsmp-client.h"
 #include "gsm-marshal.h"
 
+#include "gsm-util.h"
+#include "gsm-autostart-app.h"
 #include "gsm-manager.h"
 
-/* FIXME */
-#define GsmDesktopFile "_Gsm_DesktopFile"
+#define GsmDesktopFile "_GSM_DesktopFile"
 
 #define IS_STRING_EMPTY(x) ((x)==NULL||(x)[0]=='\0')
 
-
 #define GSM_XSMP_CLIENT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GSM_TYPE_XSMP_CLIENT, GsmXSMPClientPrivate))
 
 struct GsmXSMPClientPrivate
@@ -52,10 +53,13 @@ struct GsmXSMPClientPrivate
         guint      watch_id;
         guint      protocol_timeout;
 
-        int        current_save_yourself;
-        int        next_save_yourself;
         char      *description;
         GPtrArray *props;
+
+        /* SaveYourself state */
+        int        current_save_yourself;
+        int        next_save_yourself;
+        guint      next_save_yourself_allow_interact : 1;
 };
 
 enum {
@@ -78,25 +82,37 @@ client_iochannel_watch (GIOChannel    *channel,
                         GIOCondition   condition,
                         GsmXSMPClient *client)
 {
+        gboolean keep_going;
 
+        g_object_ref (client);
         switch (IceProcessMessages (client->priv->ice_connection, NULL, NULL)) {
         case IceProcessMessagesSuccess:
-                return TRUE;
+                keep_going = TRUE;
+                break;
 
         case IceProcessMessagesIOError:
                 g_debug ("GsmXSMPClient: IceProcessMessagesIOError on '%s'", client->priv->description);
                 gsm_client_set_status (GSM_CLIENT (client), GSM_CLIENT_FAILED);
+                /* Emitting "disconnected" will eventually cause
+                 * IceCloseConnection() to be called.
+                 */
                 gsm_client_disconnected (GSM_CLIENT (client));
-                return FALSE;
+                keep_going = FALSE;
+                break;
 
         case IceProcessMessagesConnectionClosed:
                 g_debug ("GsmXSMPClient: IceProcessMessagesConnectionClosed on '%s'",
                          client->priv->description);
-                return FALSE;
+                client->priv->ice_connection = NULL;
+                keep_going = FALSE;
+                break;
 
         default:
                 g_assert_not_reached ();
         }
+        g_object_unref (client);
+
+        return keep_going;
 }
 
 /* Called if too much time passes between the initial connection and
@@ -209,6 +225,7 @@ gsm_xsmp_client_init (GsmXSMPClient *client)
         client->priv->props = g_ptr_array_new ();
         client->priv->current_save_yourself = -1;
         client->priv->next_save_yourself = -1;
+        client->priv->next_save_yourself_allow_interact = FALSE;
 }
 
 
@@ -377,7 +394,6 @@ prop_to_command (SmProp *prop)
         return g_string_free (str, FALSE);
 }
 
-#if 0
 static char *
 xsmp_get_restart_command (GsmClient *client)
 {
@@ -405,12 +421,11 @@ xsmp_get_discard_command (GsmClient *client)
 
         return prop_to_command (prop);
 }
-#endif
 
 static void
 do_save_yourself (GsmXSMPClient *client,
                   int            save_type,
-                  gboolean       forceful)
+                  gboolean       allow_interact)
 {
         g_assert (client->priv->conn != NULL);
 
@@ -425,8 +440,12 @@ do_save_yourself (GsmXSMPClient *client,
                 g_debug ("GsmXSMPClient:   queuing new SaveYourself for '%s'",
                          client->priv->description);
                 client->priv->next_save_yourself = save_type;
+                client->priv->next_save_yourself_allow_interact = allow_interact;
         } else {
                 client->priv->current_save_yourself = save_type;
+                /* make sure we don't have anything queued */
+                client->priv->next_save_yourself = -1;
+                client->priv->next_save_yourself_allow_interact = FALSE;
 
                 switch (save_type) {
                 case SmSaveLocal:
@@ -440,7 +459,7 @@ do_save_yourself (GsmXSMPClient *client,
 
                 default:
                         /* Logout */
-                        if (forceful) {
+                        if (!allow_interact) {
                                 SmsSaveYourself (client->priv->conn,
                                                  save_type, /* save type */
                                                  TRUE, /* shutdown */
@@ -448,17 +467,16 @@ do_save_yourself (GsmXSMPClient *client,
                                                  TRUE); /* fast */
                         } else {
                                 SmsSaveYourself (client->priv->conn,
-                                                 save_type,
-                                                 TRUE,
-                                                 SmInteractStyleAny,
-                                                 FALSE);
+                                                 save_type, /* save type */
+                                                 TRUE, /* shutdown */
+                                                 SmInteractStyleAny, /* interact style */
+                                                 FALSE /* fast */);
                         }
                         break;
                 }
         }
 }
 
-#if 0
 static void
 xsmp_save_yourself_phase2 (GsmClient *client)
 {
@@ -478,7 +496,6 @@ xsmp_interact (GsmClient *client)
 
         SmsInteract (xsmp->priv->conn);
 }
-#endif
 
 static gboolean
 xsmp_cancel_end_session (GsmClient *client,
@@ -497,9 +514,197 @@ xsmp_cancel_end_session (GsmClient *client,
         }
 
         SmsShutdownCancelled (xsmp->priv->conn);
+
+        /* reset the state */
+        xsmp->priv->current_save_yourself = -1;
+        xsmp->priv->next_save_yourself = -1;
+        xsmp->priv->next_save_yourself_allow_interact = FALSE;
+
         return TRUE;
 }
 
+static char *
+get_desktop_file_path (GsmXSMPClient *client)
+{
+        SmProp     *prop;
+        char       *desktop_file_path = NULL;
+        char      **dirs;
+        const char *program_name;
+
+        /* XSMP clients using eggsmclient defines a special property
+         * pointing to their respective desktop entry file */
+        prop = find_property (client, GsmDesktopFile, NULL);
+
+        if (prop) {
+                GFile *file = g_file_new_for_uri (prop->vals[0].value);
+                desktop_file_path = g_file_get_path (file);
+                g_object_unref (file);
+                goto out;
+        }
+
+        /* If we can't get desktop file from GsmDesktopFile then we
+         * try to find the desktop file from its program name */
+        prop = find_property (client, SmProgram, NULL);
+        program_name = prop->vals[0].value;
+
+        dirs = gsm_util_get_autostart_dirs ();
+
+        desktop_file_path =
+                gsm_util_find_desktop_file_for_app_name (program_name,
+                                                         dirs);
+
+        g_strfreev (dirs);
+
+out:
+        g_debug ("GsmXSMPClient: desktop file for client %s is %s",
+                 gsm_client_peek_id (GSM_CLIENT (client)),
+                 desktop_file_path ? desktop_file_path : "(null)");
+
+        return desktop_file_path;
+}
+
+static void
+set_desktop_file_keys_from_client (GsmClient *client,
+                                   GKeyFile  *keyfile)
+{
+        SmProp *prop;
+        char   *name;
+        char   *comment;
+
+        prop = find_property (GSM_XSMP_CLIENT (client), SmProgram, NULL);
+        name = g_strdup (prop->vals[0].value);
+
+        comment = g_strdup_printf ("Client %s which was automatically saved",
+                                   gsm_client_peek_startup_id (client));
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               G_KEY_FILE_DESKTOP_KEY_NAME,
+                               name);
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               G_KEY_FILE_DESKTOP_KEY_COMMENT,
+                               comment);
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               G_KEY_FILE_DESKTOP_KEY_ICON,
+                               "system-run");
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               G_KEY_FILE_DESKTOP_KEY_TYPE,
+                               "Application");
+
+        g_key_file_set_boolean (keyfile,
+                                G_KEY_FILE_DESKTOP_GROUP,
+                                G_KEY_FILE_DESKTOP_KEY_STARTUP_NOTIFY,
+                                TRUE);
+
+        g_free (name);
+        g_free (comment);
+}
+
+static GKeyFile *
+create_client_key_file (GsmClient   *client,
+                        const char  *desktop_file_path,
+                        GError     **error) {
+        GKeyFile *keyfile;
+
+        keyfile = g_key_file_new ();
+
+        if (desktop_file_path != NULL) {
+                g_key_file_load_from_file (keyfile,
+                                           desktop_file_path,
+                                           G_KEY_FILE_KEEP_COMMENTS |
+                                           G_KEY_FILE_KEEP_TRANSLATIONS,
+                                           error);
+        } else {
+                set_desktop_file_keys_from_client (client, keyfile);
+        }
+
+        return keyfile;
+}
+
+static GsmClientRestartStyle
+xsmp_get_restart_style_hint (GsmClient *client);
+
+static GKeyFile *
+xsmp_save (GsmClient *client,
+           GError   **error)
+{
+        GsmClientRestartStyle restart_style;
+
+        GKeyFile *keyfile = NULL;
+        char     *desktop_file_path = NULL;
+        char     *exec_program = NULL;
+        char     *exec_discard = NULL;
+        char     *startup_id = NULL;
+        GError   *local_error;
+
+        g_debug ("GsmXSMPClient: saving client with id %s",
+                 gsm_client_peek_id (client));
+
+        local_error = NULL;
+
+        restart_style = xsmp_get_restart_style_hint (client);
+        if (restart_style == GSM_CLIENT_RESTART_NEVER) {
+                goto out;
+        }
+
+        exec_program = xsmp_get_restart_command (client);
+        if (!exec_program) {
+                goto out;
+        }
+
+        desktop_file_path = get_desktop_file_path (GSM_XSMP_CLIENT (client));
+
+        keyfile = create_client_key_file (client,
+                                          desktop_file_path,
+                                          &local_error);
+
+        if (local_error) {
+                goto out;
+        }
+
+        g_object_get (client,
+                      "startup-id", &startup_id,
+                      NULL);
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               GSM_AUTOSTART_APP_STARTUP_ID_KEY,
+                               startup_id);
+
+        g_key_file_set_string (keyfile,
+                               G_KEY_FILE_DESKTOP_GROUP,
+                               G_KEY_FILE_DESKTOP_KEY_EXEC,
+                               exec_program);
+
+        exec_discard = xsmp_get_discard_command (client);
+        if (exec_discard)
+                g_key_file_set_string (keyfile,
+                                       G_KEY_FILE_DESKTOP_GROUP,
+                                       GSM_AUTOSTART_APP_DISCARD_KEY,
+                                       exec_discard);
+
+out:
+        g_free (desktop_file_path);
+        g_free (exec_program);
+        g_free (exec_discard);
+        g_free (startup_id);
+
+        if (local_error != NULL) {
+                g_propagate_error (error, local_error);
+                g_key_file_free (keyfile);
+
+                return NULL;
+        }
+
+        return keyfile;
+}
+
 static gboolean
 xsmp_stop (GsmClient *client,
            GError   **error)
@@ -518,7 +723,8 @@ xsmp_query_end_session (GsmClient *client,
                         guint      flags,
                         GError   **error)
 {
-        gboolean forceful;
+        gboolean allow_interact;
+        int      save_type;
 
         if (GSM_XSMP_CLIENT (client)->priv->conn == NULL) {
                 g_set_error (error,
@@ -528,8 +734,16 @@ xsmp_query_end_session (GsmClient *client,
                 return FALSE;
         }
 
-        forceful = (flags & GSM_CLIENT_END_SESSION_FLAG_FORCEFUL);
-        do_save_yourself (GSM_XSMP_CLIENT (client), SmSaveGlobal, forceful);
+        allow_interact = !(flags & GSM_CLIENT_END_SESSION_FLAG_FORCEFUL);
+
+        /* we don't want to save the session state, but we just want to know if
+         * there's user data the client has to save and we want to give the
+         * client a chance to tell the user about it. This is consistent with
+         * the manager not setting GSM_CLIENT_END_SESSION_FLAG_SAVE for this
+         * phase. */
+        save_type = SmSaveGlobal;
+
+        do_save_yourself (GSM_XSMP_CLIENT (client), save_type, allow_interact);
         return TRUE;
 }
 
@@ -538,7 +752,7 @@ xsmp_end_session (GsmClient *client,
                   guint      flags,
                   GError   **error)
 {
-        gboolean forceful;
+        gboolean phase2;
 
         if (GSM_XSMP_CLIENT (client)->priv->conn == NULL) {
                 g_set_error (error,
@@ -548,8 +762,28 @@ xsmp_end_session (GsmClient *client,
                 return FALSE;
         }
 
-        forceful = (flags & GSM_CLIENT_END_SESSION_FLAG_FORCEFUL);
-        do_save_yourself (GSM_XSMP_CLIENT (client), SmSaveGlobal, forceful);
+        phase2 = (flags & GSM_CLIENT_END_SESSION_FLAG_LAST);
+
+        if (phase2) {
+                xsmp_save_yourself_phase2 (client);
+        } else {
+                gboolean allow_interact;
+                int      save_type;
+
+                /* we gave a chance to interact to the app during
+                 * xsmp_query_end_session(), now it's too late to interact */
+                allow_interact = FALSE;
+
+                if (flags & GSM_CLIENT_END_SESSION_FLAG_SAVE) {
+                        save_type = SmSaveBoth;
+                } else {
+                        save_type = SmSaveGlobal;
+                }
+
+                do_save_yourself (GSM_XSMP_CLIENT (client),
+                                  save_type, allow_interact);
+        }
+
         return TRUE;
 }
 
@@ -621,7 +855,10 @@ gsm_xsmp_client_disconnect (GsmXSMPClient *client)
 
         if (client->priv->conn != NULL) {
                 SmsCleanUp (client->priv->conn);
-        } else {
+        }
+
+        if (client->priv->ice_connection != NULL) {
+                IceSetShutdownNegotiation (client->priv->ice_connection, FALSE);
                 IceCloseConnection (client->priv->ice_connection);
         }
 
@@ -668,12 +905,12 @@ xsmp_get_restart_style_hint (GsmClient *client)
         GsmClientRestartStyle hint;
 
         g_debug ("GsmXSMPClient: getting restart style");
-        hint = GSM_CLIENT_RESTART_NEVER;
+        hint = GSM_CLIENT_RESTART_IF_RUNNING;
 
         prop = find_property (GSM_XSMP_CLIENT (client), SmRestartStyleHint, NULL);
 
         if (!prop || strcmp (prop->type, SmCARD8) != 0) {
-                return GSM_CLIENT_RESTART_NEVER;
+                return GSM_CLIENT_RESTART_IF_RUNNING;
         }
 
         switch (((unsigned char *)prop->vals[0].value)[0]) {
@@ -756,6 +993,7 @@ gsm_xsmp_client_class_init (GsmXSMPClientClass *klass)
         object_class->get_property         = gsm_xsmp_client_get_property;
         object_class->set_property         = gsm_xsmp_client_set_property;
 
+        client_class->impl_save                   = xsmp_save;
         client_class->impl_stop                   = xsmp_stop;
         client_class->impl_query_end_session      = xsmp_query_end_session;
         client_class->impl_end_session            = xsmp_end_session;
@@ -857,13 +1095,12 @@ register_client_callback (SmsConn    conn,
                 g_debug ("GsmXSMPClient: Sending initial SaveYourself");
                 SmsSaveYourself (conn, SmSaveLocal, False, SmInteractStyleNone, False);
                 client->priv->current_save_yourself = SmSaveLocal;
-
-                free (previous_id);
         }
 
         gsm_client_set_status (GSM_CLIENT (client), GSM_CLIENT_REGISTERED);
 
         g_free (id);
+        free (previous_id);
 
         return TRUE;
 }
@@ -925,7 +1162,7 @@ save_yourself_request_callback (SmsConn   conn,
                 g_signal_emit (client, signals[LOGOUT_REQUEST], 0, !fast);
         } else if (!shutdown && !global) {
                 g_debug ("GsmXSMPClient:   initiating checkpoint");
-                do_save_yourself (client, SmSaveLocal, FALSE);
+                do_save_yourself (client, SmSaveLocal, TRUE);
         } else {
                 g_debug ("GsmXSMPClient:   ignoring");
         }
@@ -940,21 +1177,13 @@ save_yourself_phase2_request_callback (SmsConn   conn,
         g_debug ("GsmXSMPClient: Client '%s' received SaveYourselfPhase2Request",
                  client->priv->description);
 
-        /* Treat this just like a SaveYourselfDone */
-        if (client->priv->current_save_yourself == SmSaveLocal) {
-                client->priv->current_save_yourself = -1;
-                SmsSaveComplete (client->priv->conn);
-        } else {
-                client->priv->current_save_yourself = -1;
-        }
+        client->priv->current_save_yourself = -1;
 
         /* this is a valid response to SaveYourself and therefore
            may be a response to a QES or ES */
-        gdm_client_end_session_response (GSM_CLIENT (client),
-                                         TRUE,
+        gsm_client_end_session_response (GSM_CLIENT (client),
+                                         TRUE, TRUE, FALSE,
                                          NULL);
-
-        /* FIXME: change priority so this client runs at the end? */
 }
 
 static void
@@ -963,17 +1192,20 @@ interact_request_callback (SmsConn   conn,
                            int       dialog_type)
 {
         GsmXSMPClient *client = manager_data;
+#if 0
         gboolean       res;
         GError        *error;
+#endif
 
         g_debug ("GsmXSMPClient: Client '%s' received InteractRequest(%s)",
                  client->priv->description,
                  dialog_type == SmDialogNormal ? "Dialog" : "Errors");
 
-        gdm_client_end_session_response (GSM_CLIENT (client),
-                                         FALSE,
+        gsm_client_end_session_response (GSM_CLIENT (client),
+                                         FALSE, FALSE, FALSE,
                                          _("This program is blocking log out."));
 
+#if 0
         /* Can't just call back with Interact because session client
            grabs the keyboard!  So, we try to get it to release
            grabs by telling it we've cancelled the shutdown.
@@ -985,6 +1217,8 @@ interact_request_callback (SmsConn   conn,
                 g_warning ("Unable to cancel end session: %s", error->message);
                 g_error_free (error);
         }
+#endif
+        xsmp_interact (GSM_CLIENT (client));
 }
 
 static void
@@ -998,8 +1232,8 @@ interact_done_callback (SmsConn   conn,
                  client->priv->description,
                  cancel_shutdown ? "True" : "False");
 
-        gdm_client_end_session_response (GSM_CLIENT (client),
-                                         TRUE,
+        gsm_client_end_session_response (GSM_CLIENT (client),
+                                         TRUE, FALSE, cancel_shutdown,
                                          NULL);
 }
 
@@ -1014,28 +1248,25 @@ save_yourself_done_callback (SmsConn   conn,
                  client->priv->description,
                  success ? "True" : "False");
 
-        if (client->priv->current_save_yourself == SmSaveLocal) {
-                client->priv->current_save_yourself = -1;
-                SmsSaveComplete (client->priv->conn);
-        } else {
-                client->priv->current_save_yourself = -1;
-        }
+	if (client->priv->current_save_yourself != -1) {
+		SmsSaveComplete (client->priv->conn);
+		client->priv->current_save_yourself = -1;
+	}
 
-        /* If success is false then the application still has
-           unsafe data.  We may also have tricked it into sending
-           us this message when we faked the ShutdownCancel to
-           break its grabs. */
-        if (success) {
-                gdm_client_end_session_response (GSM_CLIENT (client),
-                                                 TRUE,
-                                                 NULL);
-        }
+        /* If success is false then the application couldn't save data. Nothing
+         * the session manager can do about, though. FIXME: we could display a
+         * dialog about this, I guess. */
+        gsm_client_end_session_response (GSM_CLIENT (client),
+                                         TRUE, FALSE, FALSE,
+                                         NULL);
 
         if (client->priv->next_save_yourself) {
-                int save_type = client->priv->next_save_yourself;
+                int      save_type = client->priv->next_save_yourself;
+                gboolean allow_interact = client->priv->next_save_yourself_allow_interact;
 
                 client->priv->next_save_yourself = -1;
-                do_save_yourself (client, save_type, FALSE);
+                client->priv->next_save_yourself_allow_interact = -1;
+                do_save_yourself (client, save_type, allow_interact);
         }
 }
 
